@@ -36,6 +36,7 @@
 #include "vino-dbus-listener.h"
 #include "vino-util.h"
 #include "vino-enums.h"
+#include "vino-background.h"
 #include <sys/poll.h>
 #include <dbus/dbus-glib.h>
 #include <gtk/gtk.h>
@@ -82,6 +83,7 @@ struct _VinoServerPrivate
   guint             last_auth_failed : 1;
   guint             use_alternative_port : 1;
   guint             lock_screen : 1;
+  guint             disable_background : 1;
 };
 
 struct _VinoClient
@@ -111,7 +113,8 @@ enum
   PROP_AUTH_METHODS,
   PROP_VNC_PASSWORD,
   PROP_PORT,
-  PROP_LOCK_SCREEN
+  PROP_LOCK_SCREEN,
+  PROP_DISABLE_BACKGROUND
 };
 
 static enum rfbNewClientAction vino_server_auth_client (VinoServer *server,
@@ -166,20 +169,50 @@ vino_server_lock_screen (VinoServer *server)
 #undef GNOME_SCREENSAVER_PATH
 }
 
+void
+vino_server_set_disable_background (VinoServer *server,
+                                    gboolean disable_background)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  disable_background = disable_background != FALSE;
+
+  if (server->priv->disable_background != disable_background)
+    {
+      server->priv->disable_background = disable_background;
+
+      g_object_notify (G_OBJECT (server), "disable-background");
+    }
+}
+
+gboolean
+vino_server_get_disable_background (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
+
+  return server->priv->disable_background;
+}
 
 static void
 vino_server_client_accepted (VinoServer *server,
                              VinoClient *client)
 {
   vino_status_icon_add_client (server->priv->icon, client);
+
+  if (vino_server_get_disable_background (server))
+     vino_background_draw (FALSE);
 }
 
+
 static void
 vino_server_client_disconnected (VinoServer *server,
                                  VinoClient *client)
 {
-  vino_status_icon_remove_client (server->priv->icon, client);
-  vino_server_lock_screen (server);
+  if (vino_status_icon_remove_client (server->priv->icon, client))
+    vino_server_lock_screen (server);
+
+  if (vino_server_get_disable_background (server))
+    vino_background_draw (TRUE);
 }
 
 static void
@@ -222,10 +255,10 @@ vino_server_handle_client_gone (rfbClientPtr rfb_client)
 	  g_io_channel_unref (client->io_channel);
 	  client->io_channel = NULL;
 
-	  g_free (l->data);
 	  server->priv->clients = g_slist_delete_link (server->priv->clients, l);
 
           vino_server_client_disconnected (server, client);
+	  g_free (client);
 
 	  break;
 	}
@@ -360,7 +393,10 @@ vino_server_handle_new_client (rfbClientPtr rfb_client)
   dprintf (RFB, "New client on fd %d\n", rfb_client->sock);
 
   if (!server->priv->fb)
-    vino_server_setup_framebuffer (server);
+    {
+      vino_server_setup_framebuffer (server);
+      rfb_client->format = server->priv->rfb_screen->rfbServerFormat;
+    }
 
   client = g_new0 (VinoClient, 1);
 
@@ -481,7 +517,7 @@ vino_server_handle_clipboard_event (char         *str,
   if (server->priv->view_only)
     return;
 
-  vino_input_handle_clipboard_event (server->priv->screen, str, len);
+  vino_input_handle_clipboard_event (server->priv->screen, str, len, server);
 }
 
 static gboolean
@@ -778,6 +814,9 @@ vino_server_clipboard_cb (GtkClipboard *cb,
   if (server->priv->on_hold)
     return;
 
+  if (gtk_clipboard_get_owner (cb) == G_OBJECT (server))
+    return;
+
   text = gtk_clipboard_wait_for_text (cb);
   if (!text)
     return;
@@ -1010,6 +1049,9 @@ vino_server_set_property (GObject      *object,
     case PROP_LOCK_SCREEN:
       vino_server_set_lock_screen (server, g_value_get_boolean (value));
       break;
+    case PROP_DISABLE_BACKGROUND:
+      vino_server_set_disable_background (server, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1062,6 +1104,9 @@ vino_server_get_property (GObject    *object,
     case PROP_LOCK_SCREEN:
       g_value_set_boolean (value, server->priv->lock_screen);
       break;
+    case PROP_DISABLE_BACKGROUND:
+      g_value_set_boolean (value, server->priv->disable_background);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1234,6 +1279,18 @@ vino_server_class_init (VinoServerClass *klass)
                                                          G_PARAM_STATIC_NICK |
                                                          G_PARAM_STATIC_BLURB));
 
+  g_object_class_install_property (gobject_class,
+				   PROP_DISABLE_BACKGROUND,
+				   g_param_spec_boolean ("disable-background",
+							 "Background on server",
+							 "Disable the background image on server",
+							 FALSE,
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
+
 }
 
 GType
@@ -1615,6 +1672,7 @@ vino_server_set_lock_screen (VinoServer *server,
     }
 }
 
+
 VinoStatusIcon *
 vino_server_get_status_icon (VinoServer *server)
 {
@@ -1622,3 +1680,4 @@ vino_server_get_status_icon (VinoServer *server)
 
   return server->priv->icon;
 }
+
